import click
from rich.console import Console
from rich.table import Table
from pygments.lexers import (
    get_all_lexers
)


console = Console()


def flatten(t):
    return ','.join([x for x in t])


@click.group(name='lexers', help='Commands for working with lexers.')
def lexers_group():
    pass


@lexers_group.command(name='list', help='List all available lexers.')
def list_lexers():

    lexers = get_all_lexers()

    table = Table(title='Available Lexers')
    table.add_column('Name', style='cyan', no_wrap=True)
    table.add_column('Short Names', style='cyan', no_wrap=True)
    table.add_column('File Types', style='cyan', no_wrap=True)

    for i in lexers:
        table.add_row(i[0], flatten(i[1]), flatten(i[2]))

    console = Console()
    console.print(table)



import os
import urllib
import click
from urllib import parse, request

from pathlib import Path
from pygments.lexers import (
    get_lexer_for_filename,
    ClassNotFound,
    get_lexer_by_name, get_lexer_for_mimetype
)


def is_url(input):
    url = parse.urlparse(input)
    if url.netloc and url.scheme:
        return True


def get_content_from_url(url):

    VALID_PROTOCOLS = ['http', 'https']
    p = parse.urlparse(url).scheme
    if p not in VALID_PROTOCOLS:
        raise click.ClickException(f'{p} is not a valid http protocol.')

    try:
        with request.urlopen(url) as conn:
            content = conn.read().decode('utf-8')
            mime_type = conn.headers['content-type'].split(';')
            return content, mime_type
    except urllib.error.HTTPError as e:
        raise click.ClickException(e)


def get_content_from_file(file):

    try:

        resolved_path = os.path.expanduser(file) if file.startswith('~') else Path(file).resolve()

        with open(resolved_path, encoding='utf-8', errors='ignore') as file:
            data = file.read()
            filename = file.name

        return data, filename

    except OSError as e:
        raise click.ClickException(f'OSError: {e}')

    except FileNotFoundError:
        raise click.ClickException(f'FileNotFound: {file}')

    except UnicodeEncodeError as e:
        raise click.ClickException(f'Could not handle file encoding: {e}')


def get_lexer_from_mimetype(mimetype):

    try:
        lexer = get_lexer_for_mimetype(mimetype)
        return lexer.name
    except ClassNotFound:
        pass


def get_lexer_from_filename(filename):

    try:
        lexer = get_lexer_for_filename(filename)
        return lexer.name
    except ClassNotFound:
        pass


def get_lexer_from_name(name):

    try:
        lexer = get_lexer_by_name(name)
        return lexer.name
    except ClassNotFound:
        pass




import click
from rich.console import Console
from click_default_group import DefaultGroup
from . import commands


@click.group(cls=DefaultGroup, default='get', default_if_no_args=True)
@click.pass_context
def cli(ctx):
    ctx.obj = Console()


# Root commands
cli.add_command(commands.get)
cli.add_command(commands.version)

# Groups
cli.add_command(commands.lexers_group)
cli.add_command(commands.themes_group)



from .get_content import get
from .get_version import version

from .list_themes import themes_group
from .list_lexers import lexers_group

__all__ = ('get', 'version', 'themes_group', 'lexers_group')



import click
from rich.console import Console
from rich.table import Table
from pygments.styles import get_all_styles


@click.group(name='themes', help='Commands for working with themes.')
def themes_group():
    pass


@themes_group.command(name='list', help='List all available themes.')
def list_themes():

    styles = list(get_all_styles())
    table = Table(title='Console themes')
    table.add_column('Themes', style='cyan', no_wrap=True)

    for i in styles:
        table.add_row(i)

    console = Console()
    console.print(table)



import sys
import click
from rich.syntax import Syntax
from .command_helpers import (
    get_lexer_from_filename,
    get_lexer_from_mimetype,
    get_lexer_from_name,
    get_content_from_url,
    get_content_from_file,
    is_url
)


@click.command(name='get',
               help='Perform syntax highlighting on raw text from a local file or a url.',
               no_args_is_help=True
               )
@click.argument('path')
@click.option('--theme',
              '-t',
              default='native',
              envvar='CATZ_THEME',
              help='''Override the default syntaxt highlighting theme.
              You can use catz themes list to view a list of available themes.'''
              )
@click.option('--lexer',
              '-l',
              default=None,
              envvar='CATZ_LEXER',
              help='''Override the lexer used when applying syntax highlighting.
              You can use catz lexers list to view a list of available lexers.'''
              )
@click.option('--highlight',
              '-hl',
              default=None,
              help='''Highlight specific lines in the parsed file.
              Accepts a comma separated list of line numbers.
              '''
              )
@click.option('--passthru',
              '-p',
              is_flag=True,
              envvar='CATZ_PASSTHRU',
              help='''Pass the content of the file directly to stdout with no lexer applied.'''
              )
@click.pass_obj
def get(console, path, theme, lexer, highlight, passthru):

    data, lexer_identifier = get_content_from_url(
        path) if is_url(path) else get_content_from_file(path)

    if passthru:
        print(data, file=sys.stdout)
    else:
        lexer_name = get_lexer_from_name(lexer) if lexer is not None else get_lexer_from_mimetype(
            lexer_identifier) if is_url(path) else get_lexer_from_filename(lexer_identifier)

        syntax_params = dict(
            code=data,
            lexer_name=lexer_name,
            theme=theme,
            line_numbers=True
        )

        if highlight is not None:

            try:
                values = highlight.split(',')
                mapped = map(int, values)
                syntax_params['highlight_lines'] = set(mapped)
            except ValueError as e:
                raise click.ClickException(f'Invalid value for --highlight / -hl: {e} is not a valid integer')

        console.print(Syntax(**syntax_params))



import os
from setuptools import setup


PRODUCT_NAME = 'catz'
DEPENDENCIES = [
    'rich==9.10.0',
    'click==7.1.2',
    'click-default-group==1.2.2'
]
VERSION = os.environ.get('GITVERSION_MAJORMINORPATCH', '0.0.3')

setup(
    name=PRODUCT_NAME,
    version=VERSION,
    author='chelnak',
    description='A colourful syntax highlighting tool for your terminal.',
    url='https://github.com/chelnak/catz',
    packages=['catz', 'catz.commands'],
    entry_points={
        'console_scripts': ['catz = catz.app:cli']
    },
    install_requires=DEPENDENCIES,
    test_suite='nose.collector',
    tests_require=['nose'],
)



import click
import pkg_resources


@click.command(name='version', help='Display version info for catz')
def version():
    version = pkg_resources.require('catz')[0].version
    print(f'catz version: {version}')
